using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class GasBehaviour : MonoBehaviour
    {
        public float wind = .025f;
        public float lifeTime = 20f;
        public float concentration = 1f;
        float staticTime;
        public Rigidbody2D rb;
        public Vector3 scale;
        public Vector3 initialScale;

        // Properties

        public GasProperties prop;
        //public Gas gas;

        
        void Start()
        {
            rb = GetComponent<Rigidbody2D>();
            staticTime = lifeTime;
            GetComponent<SpriteRenderer>().sortingLayerName = "Top";
            GetComponent<SpriteRenderer>().sortingOrder = -5;
            StartCoroutine(Check());
            StartCoroutine(Decay());
            GetComponent<SpriteRenderer>().color = prop.color;
        }
        void Update()
        {
            float windForce = wind * Time.timeScale;
            float floatForce = prop.buoyancy * Time.timeScale;
            rb.AddForce(new Vector3(wind, floatForce / 40));
            rb.AddTorque(windForce / 100);
        }
        void OnTriggerStay2D(Collider2D col)
        {
            if (col.isTrigger)
                return;
            LimbBehaviour limb = col.GetComponent<LimbBehaviour>();
            if (limb && !limb.IsAndroid)
            {
                GasIgnore ignore = col.GetComponent<GasIgnore>();
                PersonBehaviour person = limb.Person;
                if (ignore.skinProt)
                    return;
                if (limb.IsZombie && prop.type == GasProperties.GasType.Zombie)
                    return;
                limb.Damage(prop.lethality * concentration);

                float acidrng = UnityEngine.Random.value;
                float acidDamage = prop.skinDamageRate;
                if (limb.SkinMaterialHandler.AcidProgress >= prop.skinDamageClamp)
                {
                    if (!prop.clamp)
                        acidDamage /= 15f;
                    else
                        acidDamage = 0;
                }
                    
                if (acidrng > limb.SkinMaterialHandler.AcidProgress && limb.SkinMaterialHandler.AcidProgress < 1)
                    limb.SkinMaterialHandler.AcidProgress += Time.deltaTime * (acidDamage * concentration);

                float burnDamage = prop.burnDamageRate;
                if (limb.PhysicalBehaviour.BurnProgress >= prop.burnDamageClamp)
                    burnDamage /= 100;
                if (limb.PhysicalBehaviour.BurnProgress < 1)
                    limb.PhysicalBehaviour.BurnProgress += Time.deltaTime * (burnDamage * concentration);

                if (prop.type != GasProperties.GasType.Zombie)
                    return;
                if (limb.Health <= 0 && !limb.IsZombie)
                {
                    limb.IsZombie = true;
                    if (limb.HasBrain)
                    {
                        foreach (LimbBehaviour lim in limb.Person.Limbs)
                        {
                            lim.IsZombie = true;
                        }
                    }
                }
            }
        }
        void OnTriggerEnter2D(Collider2D col)
        {
            if (col.isTrigger)
                return;
            GasIgnore ignore = col.GetComponent<GasIgnore>();
            if (ignore.inhaleProt)
                return;
            BreatheBehaviour breath = col.GetComponent<BreatheBehaviour>();
            //GasBehaviour gas = col.GetComponent<GasBehaviour>();
            if (breath && !breath.currentGas)
                breath.currentGas = this;
        }
        void OnTriggerExit2D(Collider2D col)
        {
            BreatheBehaviour breath = col.GetComponent<BreatheBehaviour>();
            //GasBehaviour gas = col.GetComponent<GasBehaviour>();
            if (breath && breath.currentGas == this)
                breath.currentGas = null;
        }
        public IEnumerator Decay()
        {
            while (lifeTime > 0)
            {
                rb.transform.localScale = scale + new Vector3(staticTime / 2f - lifeTime / 2f, staticTime / 2f - lifeTime / 2f);
                Color col = GetComponent<SpriteRenderer>().color;
                col.a = lifeTime / (staticTime * 1.5f);
                concentration = lifeTime / staticTime;
                GetComponent<SpriteRenderer>().color = col;
                lifeTime -= Time.deltaTime;
                yield return null;
            }
            Destroy(gameObject);
        }
        public IEnumerator Check()
        {
            while (1 == 1)
            {
                CollisionCheck();
                yield return new WaitForSeconds(.5f);
            }
        }
        public void CollisionCheck()
        {
            float upDir = 2f;
            float rightDir = 2f;
            if (wind < 0)
                rightDir = -2f;
            if (prop.buoyancy < 0)
                upDir *= -2f;
            RaycastHit2D hitUp = Physics2D.Raycast(transform.position, Vector3.up, upDir, ~(1 << 10));
            RaycastHit2D hitRight = Physics2D.Raycast(transform.position, Vector3.right, rightDir, ~(1 << 10));

            float mult = 10f;
            if (hitUp)
            {
                if (hitUp.transform.gameObject.GetComponent<GasBehaviour>() || hitUp.transform.gameObject.GetComponent<LimbBehaviour>() || hitUp.transform.gameObject.GetComponent<ArmorBehaviour>())
                    return;
                var vel = rb.velocity;
                float storedV = vel.y * mult;
                vel.y = 0;
                rb.velocity = vel;
                rb.AddForce(new Vector3(0, -storedV));
            }
            if (hitRight)
            {
                if (hitRight.transform.gameObject.GetComponent<GasBehaviour>() || hitRight.transform.gameObject.GetComponent<LimbBehaviour>() || hitRight.transform.gameObject.GetComponent<ArmorBehaviour>())
                    return;
                var vel = rb.velocity;
                float storedV = vel.x * mult;
                vel.x = 0;
                rb.velocity = vel;
                rb.AddForce(new Vector3(-storedV, 0));
                Debug.Log(hitRight.transform.gameObject.layer.ToString());
            }
        }
    }

    public struct GasProperties
    {
        // Lethality properties
        public float lethality;
        // Visual damage properties
        public float skinDamageRate;
        public float skinDamageClamp;
        public float burnDamageClamp;
        public float burnDamageRate;
        public bool clamp;
        // Internal damage
        public float lungDamage;
        public float consciousnessDamage;
        public float strengthDamage;
        // Other properties
        public float buoyancy;
        public float windMin;
        public float windMax;
        public float lifeMin;
        public float lifeMax;
        public enum GasType {Chlorine, Mustard, Zombie, Cyanide, Knockout, Phosphorus}
        public GasType type;
        public Color color;
        public void SetGasProperties(GasType gasType)
        {
            if (gasType == GasType.Chlorine)
            {
                skinDamageRate = .005f;
                skinDamageClamp = .4f;
                clamp = true;
                lungDamage = .1f;
                consciousnessDamage = .01f;
                type = gasType;
                color = new Color(.82f, 1, .5f);
            }
            if (gasType == GasType.Mustard)
            {
                skinDamageRate = .01f;
                skinDamageClamp = .6f;
                strengthDamage = .35f;
                lethality = .005f;
                lungDamage = .2f;
                consciousnessDamage = .1f;
                type = gasType;
                color = new Color(1, 1, .45f);
            }
            if (gasType == GasType.Zombie)
            {
                skinDamageRate = .01f;
                skinDamageClamp = .6f;
                lungDamage = .5f;
                type = gasType;
                color = new Color(.41f, .74f, .18f);
            }
            if (gasType == GasType.Knockout)
            {
                skinDamageRate = 0;
                skinDamageClamp = 0;
                lungDamage = 0;
                consciousnessDamage = .5f;
                type = gasType;
                color = new Color(.5f, 1f, 1f);
            }
            if (gasType == GasType.Cyanide)
            {
                skinDamageRate = .005f;
                skinDamageClamp = .1f;
                lethality = .01f;
                lungDamage = .005f;
                consciousnessDamage = .15f;
                type = gasType;
                buoyancy = .025f;
                color = new Color(1f, 1f, 1f);
            }
            if (gasType == GasType.Phosphorus)
            {
                skinDamageRate = .005f;
                skinDamageClamp = .6f;
                burnDamageClamp = .45f;
                strengthDamage = .15f;
                burnDamageRate = .01f;
                lethality = .005f;
                lungDamage = .5f;
                consciousnessDamage = .02f;
                type = gasType;
                color = new Color(1, 1, 1f);
            }
        }
        public void DefaultProperties(bool defaultWind, bool defaultLife, bool defaultBuoy)
        {
            if (defaultWind)
            {
                windMin = 0;
                windMax = .2f;
            }
            if (defaultLife)
            {
                lifeMin = 10;
                lifeMax = 20;
            }
            if (defaultBuoy)
            {
                buoyancy = -.25f;
            }
        }
    }
    public class Gas : ScriptableObject
    {
        public float buoyancy = -0.25f;
        public float viscocity;
        public Color color;

        public float windMin = 0;
        public float windMax = .2f;
        public float lifeMin = 10;
        public float lifeMax = 20;
    }
}