using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class Mod
    {
        public static void Main()
        {
            ModAPI.OnItemSpawned += (sender, obj) =>
            {
                //ModAPI.Notify("Object is " + obj.ToString());
                GameObject gameobject = obj.Instance;
                PersonBehaviour person = gameobject.GetComponent<PersonBehaviour>();
                if (person)
                {
                    foreach (LimbBehaviour limb in person.Limbs)
                    {
                        if (!limb.gameObject.GetComponent<GasIgnore>())
                            limb.gameObject.AddComponent<GasIgnore>();
                        if (limb.RoughClassification == LimbBehaviour.BodyPart.Head && !limb.IsAndroid)
                        {
                            if (!limb.gameObject.GetComponent<BreatheBehaviour>())
                                limb.gameObject.AddComponent<BreatheBehaviour>();
                            BreatheBehaviour breath = limb.gameObject.GetComponent<BreatheBehaviour>();
                            breath.limb = limb;
                        }
                    }
                }
            };
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Gas Canister - ChemW", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "A - ChemW",
                    DescriptionOverride = "Canister of human suffering.", //new item description
                    CategoryOverride = ModAPI.FindCategory("Explosives"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("chlorineView.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("ChlorineCanister.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        if (!Instance.GetComponent<GasEmitter>())
                            Instance.AddComponent<GasEmitter>();
                        GasEmitter gas = Instance.GetComponent<GasEmitter>();
                        gas.gasRate = .35f;
                        gas.offset = new Vector3(0, .5f);

                        gas.sounds[0] = ModAPI.LoadSound("steamstart.mp3");
                        gas.sounds[1] = ModAPI.LoadSound("steamloop.mp3");
                        gas.sounds[2] = ModAPI.LoadSound("steamend.mp3");
                        gas.gasSprite = ModAPI.LoadSprite("gas.png");

                        Instance.FixColliders();

                        GasProperties properties = new GasProperties();
                        
                        properties.SetGasProperties(GasProperties.GasType.Chlorine);
                        properties.DefaultProperties(true, true, true);

                        gas.properties = properties;
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Gas Grenade - ChemW", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "A1 - ChemW",
                    DescriptionOverride = "Spreads selected gas.", //new item description
                    CategoryOverride = ModAPI.FindCategory("Explosives"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("grenadeView.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("GasGrenade.png"); //get the SpriteRenderer and replace its sprite with a custom one
                        if (!Instance.GetComponent<GasEmitter>())
                            Instance.AddComponent<GasEmitter>();
                        GasEmitter gas = Instance.GetComponent<GasEmitter>();
                        gas.gasRate = .35f;
                        gas.fuseTime = 5f;
                        gas.offset = new Vector3(0, .15f);
                        gas.gasScale = new Vector3(.5f, .5f);

                        gas.sounds[0] = ModAPI.LoadSound("steamstart.mp3");
                        gas.sounds[1] = ModAPI.LoadSound("steamloop.mp3");
                        gas.sounds[2] = ModAPI.LoadSound("steamend.mp3");
                        gas.gasSprite = ModAPI.LoadSprite("gas.png");

                        Instance.FixColliders();

                        GasProperties properties = new GasProperties();
                        
                        properties.SetGasProperties(GasProperties.GasType.Chlorine);
                        properties.DefaultProperties(true, true, true);

                        gas.properties = properties;
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Gas Mask - ChemW", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "A - ChemW",
                    DescriptionOverride = "Prevents breathing deadly gases. Doesn't protect the skin.", //new item description
                    CategoryOverride = ModAPI.FindCategory("Entities"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("gasmaskView.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("gasmask.png"); //get the SpriteRenderer and replace its sprite with a custom one

                        if (!Instance.GetComponent<ArmorBehaviour>())
                            Instance.AddComponent<ArmorBehaviour>();

                        ArmorBehaviour armor = Instance.GetComponent<ArmorBehaviour>();
                        armor.gasInhaleProtection = true;

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 2;
                        prop.scaleOffset = new Vector3(.01f, .01f);
                        prop.offset = new Vector3(.02f, .015f);
                        prop.sprite = "gasmask.png";

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[0];
                        
                        Instance.FixColliders();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Rod"), //item to derive from
                    NameOverride = "Hazmat Suit - ChemW", //new item name with a suffix to assure it is globally unique
                    NameToOrderByOverride = "A1 - ChemW",
                    DescriptionOverride = "Prevents breathing and skin contact with deadly gases. Covers the entire body.", //new item description
                    CategoryOverride = ModAPI.FindCategory("Entities"), //new item category
                    ThumbnailOverride = ModAPI.LoadSprite("hazmatView.png"), //new item thumbnail (relative path)
                    AfterSpawn = (Instance) => //all code in the AfterSpawn delegate will be executed when the item is spawned
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("hazmathat.png"); //get the SpriteRenderer and replace its sprite with a custom one

                        int PartCount = 9;

                        if (!Instance.GetComponent<ArmorBehaviour>())
                            Instance.AddComponent<ArmorBehaviour>();

                        ArmorBehaviour armor = Instance.GetComponent<ArmorBehaviour>();
                        armor.gasInhaleProtection = true;

                        ArmorProperties prop = new ArmorProperties();
                        
                        prop.armorPiece = "Head";
                        prop.armorTier = 2;
                        prop.scaleOffset = new Vector3(.01f, .01f);
                        prop.sprite = "hazmathat.png";
                        prop.gasSkinProtection = true;

                        armor.prop = prop;
                        armor.SetProperties();

                        ArmorProperties[] armProp = new ArmorProperties[PartCount - 1];

                        armProp[0].armorPiece = "UpperBody";
                        armProp[0].armorTier = 2;
                        armProp[0].sprite = "hazmatupper.png";
                        armProp[0].scaleOffset = new Vector3(.01f, .01f);
                        armProp[0].gasSkinProtection = true;

                        armProp[1].armorPiece = "MiddleBody";
                        armProp[1].armorTier = 2;
                        armProp[1].sprite = "hazmatmid.png";
                        armProp[1].scaleOffset = new Vector3(.01f, .01f);
                        armProp[1].gasSkinProtection = true;

                        armProp[2].armorPiece = "LowerBody";
                        armProp[2].armorTier = 2;
                        armProp[2].sprite = "hazmatlower.png";
                        armProp[2].scaleOffset = new Vector3(.01f, .01f);
                        armProp[2].gasSkinProtection = true;

                        armProp[3].armorPiece = "UpperArm";
                        armProp[3].armorTier = 2;
                        armProp[3].sprite = "hazmatshoulder.png";
                        armProp[3].scaleOffset = new Vector3(.01f, .01f);
                        armProp[3].gasSkinProtection = true;
                        armProp[3].clone = true;

                        armProp[4].armorPiece = "LowerArm";
                        armProp[4].armorTier = 2;
                        armProp[4].sprite = "hazmatarm.png";
                        armProp[4].scaleOffset = new Vector3(.01f, .01f);
                        armProp[4].offset = new Vector3(0, .005f);
                        armProp[4].gasSkinProtection = true;
                        armProp[4].clone = true;

                        armProp[5].armorPiece = "UpperLeg";
                        armProp[5].armorTier = 2;
                        armProp[5].sprite = "hazmatthigh.png";
                        armProp[5].scaleOffset = new Vector3(.01f, .01f);
                        armProp[5].gasSkinProtection = true;
                        armProp[5].clone = true;

                        armProp[6].armorPiece = "LowerLeg";
                        armProp[6].armorTier = 2;
                        armProp[6].sprite = "hazmatleg.png";
                        armProp[6].scaleOffset = new Vector3(.01f, .01f);
                        armProp[6].gasSkinProtection = true;
                        armProp[6].clone = true;

                        armProp[7].armorPiece = "Foot";
                        armProp[7].armorTier = 2;
                        armProp[7].sprite = "hazmatboot.png";
                        armProp[7].scaleOffset = new Vector3(.01f, .01f);
                        armProp[7].gasSkinProtection = true;
                        armProp[7].clone = true;

                        if (armor.spawn)
                        {
                            armor.SetPieces = new ArmorBehaviour[PartCount - 1];
                            armor.SpawnOtherParts(armProp);
                        }
                        Instance.FixColliders();
                    }
                }
            );
        }
    }
}